require_relative '../../spec_helper'

describe "Encoding::UndefinedConversionError#source_encoding" do
  it "needs to be reviewed for spec completeness"
end

describe "Encoding::InvalidByteSequenceError#source_encoding" do
  it "needs to be reviewed for spec completeness"
end
