require_relative '../../spec_helper'

describe "Encoding::UndefinedConversionError#destination_encoding_name" do
  it "needs to be reviewed for spec completeness"
end

describe "Encoding::InvalidByteSequenceError#destination_encoding_name" do
  it "needs to be reviewed for spec completeness"
end
