require_relative '../../../spec_helper'

describe "Process::Tms#cutime" do
  it "needs to be reviewed for spec completeness"
end

describe "Process::Tms#cutime=" do
  it "needs to be reviewed for spec completeness"
end
