require_relative '../../../spec_helper'
require 'cgi'

describe "CGI::QueryExtension#cookies" do
  it "needs to be reviewed for spec completeness"
end

describe "CGI::QueryExtension#cookies=" do
  it "needs to be reviewed for spec completeness"
end
