#!/usr/bin/env ruby -w
# encoding: UTF-8
# frozen_string_literal: false

# ts_all.rb
#
# Created by James Edward Gray II on 2005-10-31.

require "test/unit"

require "test_csv_parsing"
require "test_features"
require "test_interface"
require "test_csv_writing"
require "test_data_converters"
require "test_row"
require "test_table"
require "test_headers"
require "test_encodings"
