/* -*- mode:c; style:ruby; coding: utf-8; indent-tabs-mode: nil -*- */

%# Copyright (c) 2017 Urabe, Shyouhei.  All rights reserved.
%#
%# This file is a part of  the programming language Ruby.  Permission is hereby
%# granted, to either  redistribute and/or modify this file,  provided that the
%# conditions mentioned  in the  file COPYING  are met.   Consult the  file for
%# details.
% raise ':FIXME:TBW' if RubyVM::VmOptsH['INSTRUCTIONS_UNIFICATION']
% n = RubyVM::Instructions.size
<%= render 'copyright' %>
<%= render 'notice', locals: {
    this_file: 'is for threaded code',
    edit: __FILE__,
} -%>

/* Let .bss section automatically initialize this variable */
/* cf. Section 6.7.8 of ISO/IEC 9899:1999 */
static const int *const *const unified_insns_data[<%= n %>];

ASSERT_VM_INSTRUCTION_SIZE(unified_insns_data);
