require_relative '../../../spec_helper'

describe "Process::Tms#cstime" do
  it "needs to be reviewed for spec completeness"
end

describe "Process::Tms#cstime=" do
  it "needs to be reviewed for spec completeness"
end
