%# -*- mode:c; style:ruby; coding: utf-8; indent-tabs-mode: nil -*-
%# Copyright (c) 2017 Urabe, Shyouhei.  All rights reserved.
%#
%# This file is a part of  the programming language Ruby.  Permission is hereby
%# granted, to either  redistribute and/or modify this file,  provided that the
%# conditions mentioned  in the  file COPYING  are met.   Consult the  file for
%# details.
%;
% if expr.blank?
%   # empty
% elsif ! expr.__LINE__
<%= expr.expr %>
% else
#line <%= expr.__LINE__ %> <%=cstr expr.__FILE__ %>
<%= expr.expr %>
#pragma RubyVM reset source
% end
